@@ -12,8 +12,8 @@ short.
 """
 
 
-__version__ = '0.15.2'
-__author__ = 'kaliko jack'
+__version__ = '0.16.1'
+__author__ = 'kaliko'
 __email__ = 'kaliko@azylum.org'
 __url__ = 'git://git.kaliko.me/sima.git'
 
